package answer

import (
	"encoding/json"
	"log/slog"
	"net/url"
	"strings"

	gollm "gitea.stevedudenhoeffer.com/steve/go-llm"
)

type searchResults struct {
	Url    string `json:"url"`
	Answer string `json:"answer"`
}

func (s searchResults) String() (string, error) {
	b, err := json.Marshal(s)
	if err != nil {
		return "", err
	}

	return string(b), nil
}

func internalSearch(ctx *gollm.Context, q Question, searchTerm string) (searchResults, error) {
	slog.Info("searching", "search", searchTerm, "question", q)
	results, err := q.Search.Search(ctx, searchTerm)
	if err != nil {
		return searchResults{}, err
	}

	if len(results) == 0 {
		return searchResults{Url: "not-found", Answer: "no search results found"}, nil
	}

	// first pass try to see if any provide the result without needing archive
	for _, r := range results {
		trimmed := strings.TrimSpace(r.URL)
		if trimmed == "" {

		}

		slog.Info("extracting article", "url", trimmed)
		u, err := url.Parse(trimmed)
		if err != nil {
			continue
		}

		a, err := extractArticle(ctx, q.Cache, u)

		if err != nil {
			continue
		}

		slog.Info("extracted article", "url", a.URL, "title", a.Title, "body", a.Body)

		if a.Title != "" && a.Body != "" {
			answer, err := doesTextAnswerQuestion(ctx, q, a.Body)

			if err != nil {
				slog.Error("error checking if text answers question", "question", q.Question, "error", err)
				continue
			}

			if answer != "" {
				return searchResults{Url: u.String(), Answer: answer}, nil
			}
		}
	}

	return searchResults{Url: "not-found", Answer: "no searched results answered"}, nil
}

func functionSearch(ctx *gollm.Context, q Question, searchTerm string) (string, error) {
	res, err := internalSearch(ctx, q, searchTerm)
	if err != nil {
		return "", err
	}

	return res.String()
}
